# -*- coding: utf-8 -*-

""" Module

lpom -- Local Partial Order Model.

LPOM0 and LPOMext are approximations to calculate the
average height, hav, of partially ordered sets (posets).

"""
name = "pyhasse.lpom"

__author__ = """Rainer Bruggemann"""
__email__ = 'rainer.bruggemann@pyhasse.org'
__version__ = '0.2.1'



# -*- coding: utf-8 -*-


class LPOM(object):
    """A Class calculating averaged height according to the LPOM model.

    Three variants:

    LPOM0:
      with the resulting variable self.hav LPOMbalance with the
      resulting variable self.hprob and LPOMext with the resulting
      variable self.havext

      Basis for all three variants is the concept of sets, generated by
      a single object (therefore "local partial order model".

    LPOM0:
      is the only important set Incomp(x), the set of all
      objects incomparable with x In LPOMbalance the relevant local sets
      are the down(set) and the up(set) generated by x In

    LPOMext:
      all three sets are important.

    """

    def __init__(self, precision=2):
        self.hav = []
        self.precision = precision

    def calc_hav(self, downset, incompset, rred):
        """ calculates the averaged height according to the LPOM0 model.

        :param downset: elements in the principal downset
                        of repres. elements
        :type downset: list

        :param incompset: elements incomparable
                          with a certain element
        :type incompset: list

        :param rred: representative elements
        :type rred: number

        :returns: evaeraged height
        :rtype: list

        """

        self.hav = []
        for i in range(0, rred):
            d = len(downset[i])
            lic = len(incompset[i])
            denom = 1.0 * rred + 1.0 - 1.0 * lic
            self.hav.append(
                round(1.0 * d * (rred + 1) / denom,
                      self.precision))
        return self.hav

    def calc_hav_balance(self, matrix, csv, downset, upset, rred):
        """ calculates average height by a simple balance concept

        called "LPOMbalance". The idea: estimate the mutual probability,
        mutprob(x,y) that an object x > object y, based on the set of all
        linear extensions.

        mutprob(x,x) is set 1.

        hprob(i), the average height of the ith object, due to
        LPOMbalance = sum mutprob(i,j), summing over j.

        :param downset: elements in the downsets
                        of repres. elements
        :type downset: list

        :param upset: elements in the upsets
                      of repres. elements
        :type upset: list

        :param rred: representative elements
        :type rred: list

        :returns:  average height by a simple balance concept
        :rtype: list

        """

        mutprob = []
        self.hprob = []
        Q = []
        for i in range(0, rred):
            Q.append(1.0 * len(downset[i]) / (1.0 * len(upset[i])))

        for i1 in range(0, rred):
            mutprob.append(0)
            mutprob[i1] = []
            for i2 in range(0, rred):
                denomQ = 1.0 * Q[i1] + 1.0 * Q[i2]
                if i1 == i2:
                    mutprob[i1].append(1.0)
                else:
                    mutprob[i1].append(1.0 * Q[i1] / denomQ)

        for i1 in range(0, rred):
            self.hprob.append(0)
            for i2 in range(0, rred):
                self.hprob[i1] += mutprob[i1][i2]

        return self.hprob

    def calc_hav_ext(self, downset, upset, incompset, rred):
        """ calculates the averaged height according to the LPOMext model.

        :param downset: elements in the downsets
                        of repres. elements
        :type downset: list

        :param upset: elements in the upsets
                      of repres. elements
        :type upset: list

        :param incompset: elements incomparable
                          with each repres. element
        :type incompset: list
        :param rred: representative elements
        :type rred: list

        :var upset: elements in the principal upset
                    of repres. elements
        :type upset: list

        :var pdown: counts the elements in downset
        :type pdown: int

        :var pup: counts the elements in upset
        :type pupp: int

        :returns: averaged height according to the LPOMext model
        :rtype: list

        """
        self.havext = []
        for i in range(0, rred):
            self.havext.append(len(downset[i]))
        for i1 in range(0, rred):
            inc = 0
            for i2 in incompset[i1]:
                pdown = 0
                pup = 0
                for i3 in incompset[i2]:
                    if i3 in downset[i1]:
                        pdown += 1
                for i3 in incompset[i2]:
                    if i3 in upset[i1]:
                        pup += 1
                if (pdown + pup) != 0:
                    inc += 1.0 * pdown / (1.0 * pdown + 1.0 * pup)
                else:
                    inc += 0
            self.havext[i1] += inc
        for i1 in range(0, rred):
            self.havext[i1] = round(self.havext[i1], self.precision)
        return self.havext



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requires = [
    'setuptools',
    'pyhasse.core'
]


setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Rainer Bruggemann",
    author_email='rainer.bruggemann@pyhasse.org',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.9',
    ],
    description="""pyhasse-lpom: Synthetic indicator, without the need of weights for single indicators of a multi-indicator system""",
    install_requires=requires,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    keywords='lpom',
    packages=['pyhasse.lpom'],
    namespace_packages=['pyhasse'],
    name='pyhasse.lpom',
    package_dir={'': 'src'},
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://pyhasse.org',
    version='0.2.1',
    zip_safe=False,
)
